<?php
require_once 'config.php';
require_once 'db.php';

// Set the content type to JSON
header('Content-Type: application/json');

// Check if the request method is POST
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Get the tile data from the request
    $tileData = json_decode(file_get_contents('php://input'), true);

    // Validate the input
    if (isset($tileData['tx']) && isset($tileData['ty']) && isset($tileData['image'])) {
        $tx = intval($tileData['tx']);
        $ty = intval($tileData['ty']);
        $imageData = $tileData['image'];

        // Decode the base64 image data
        $image = base64_decode($imageData);

        // Create a unique filename for the tile
        $filename = "tile_{$tx}_{$ty}.png";
        $filePath = "../public/tiles/{$filename}";

        // Save the image to the server
        if (file_put_contents($filePath, $image) !== false) {
            // Save tile information to the database
            $db = new Database();
            $db->saveTile($tx, $ty, $filename);

            // Respond with success
            echo json_encode(['status' => 'success', 'message' => 'Tile uploaded successfully.']);
        } else {
            echo json_encode(['status' => 'error', 'message' => 'Failed to save tile image.']);
        }
    } else {
        echo json_encode(['status' => 'error', 'message' => 'Invalid input.']);
    }
} else {
    echo json_encode(['status' => 'error', 'message' => 'Invalid request method.']);
}
?>