const TILE_SIZE = 512;   // Each tile is 512×512 pixels
const LINE_WIDTH = 4;    // Thickness of drawn lines
const mainCanvas = document.getElementById('mainCanvas');
const ctx = mainCanvas.getContext('2d');

// Set initial canvas size
let width = window.innerWidth;
let height = window.innerHeight;
mainCanvas.width = width;
mainCanvas.height = height;

// Global offset in 'world coordinates'
let offset = { x: 0, y: 0 };

// Data structure for storing all lines.
let lines = [];

// Temporary array to collect the current line’s points during drawing
let currentLine = [];

// Booleans to track user’s interaction states
let isDrawing = false;
let isPanning = false;   // Right-click panning
let lastMousePos = { x: 0, y: 0 };

// A map from tile key ("tx,ty") to an off-screen canvas
let tileCache = {};

// RESTful server address
const SERVER_URL = 'http://your-server-address/upload_tile.php';

/**********************************************************************
 * Utility Functions
 **********************************************************************/

function getTileIndices(x, y) {
  return {
    tx: Math.floor(x / TILE_SIZE),
    ty: Math.floor(y / TILE_SIZE)
  };
}

function tileKey(tx, ty) {
  return `${tx},${ty}`;
}

function uploadTile(tx, ty, tileData) {
  fetch(SERVER_URL, {
    method: 'POST',
    body: JSON.stringify({
      tx: tx,
      ty: ty,
      tileData: tileData
    }),
    headers: {
      'Content-Type': 'application/json'
    }
  })
  .then(response => response.json())
  .then(data => {
    console.log('Tile uploaded successfully:', data);
  })
  .catch(error => {
    console.error('Error uploading tile:', error);
  });
}

// Add event listeners for mouse interactions
mainCanvas.addEventListener('mousedown', (event) => {
  if (event.button === 0) { // Left mouse button
    isDrawing = true;
    currentLine.push({ x: event.clientX + offset.x, y: event.clientY + offset.y });
  } else if (event.button === 2) { // Right mouse button
    isPanning = true;
    lastMousePos = { x: event.clientX, y: event.clientY };
  }
});

mainCanvas.addEventListener('mousemove', (event) => {
  if (isDrawing) {
    currentLine.push({ x: event.clientX + offset.x, y: event.clientY + offset.y });
    drawLine(currentLine);
  } else if (isPanning) {
    offset.x += event.clientX - lastMousePos.x;
    offset.y += event.clientY - lastMousePos.y;
    lastMousePos = { x: event.clientX, y: event.clientY };
    redrawCanvas();
  }
});

mainCanvas.addEventListener('mouseup', (event) => {
  if (isDrawing) {
    isDrawing = false;
    const { tx, ty } = getTileIndices(currentLine[0].x, currentLine[0].y);
    const tileData = ctx.getImageData(tx * TILE_SIZE, ty * TILE_SIZE, TILE_SIZE, TILE_SIZE);
    uploadTile(tx, ty, tileData);
    currentLine = [];
  } else if (isPanning) {
    isPanning = false;
  }
});

mainCanvas.addEventListener('contextmenu', (event) => {
  event.preventDefault(); // Prevent the context menu from appearing
});

// Function to draw a line on the canvas
function drawLine(line) {
  ctx.beginPath();
  ctx.moveTo(line[0].x, line[0].y);
  for (let point of line) {
    ctx.lineTo(point.x, point.y);
  }
  ctx.lineWidth = LINE_WIDTH;
  ctx.strokeStyle = 'black';
  ctx.stroke();
}

// Function to redraw the entire canvas
function redrawCanvas() {
  ctx.clearRect(0, 0, mainCanvas.width, mainCanvas.height);
  for (let line of lines) {
    drawLine(line);
  }
}